==> ninix-aya-3.6.1.tgz: ninix-aya-3.6.1/lib/ninix-update.py <==
#!/usr/bin/env python
# -*- coding: ascii -*-
#
#  ninix-update.py - a command-line network update utility for ninix
#  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
#  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
#  Copyright (C) 2002-2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
#  $Id: ninix-update.py,v 1.14 2005/08/09 00:41:52 shy Exp $
#

import getopt
import os
import sys
import tempfile
import StringIO

import codecs
import locale
locale.setlocale(locale.LC_ALL, '')

if os.environ.has_key('DISPLAY'):
    del os.environ['DISPLAY']

import ninix.home
import ninix.update
import ninix.dll

PROGRAM_NAME = os.environ.get('NINIX_UPDATE', sys.argv[0])

USAGE = '''\
Usage: %s [options] [URL] [directory]
Options:
  -l, --list          show a list of ghosts
  -s, --script        show a list of ghosts as a shell script
  -H, --homedir DIR   ninix home directory (default: ~/.ninix)
  -T, --tempdir DIR   temporary directory (default: /usr/tmp)
  -q, --quiet         suppress most messages
  -h, --help          show this message
'''

def usage():
    sys.stderr.write(USAGE % os.path.basename(PROGRAM_NAME))
    sys.exit(1)

def main():
    try:
        options, argv = getopt.getopt(sys.argv[1:], 'lsH:T:qh',
            ['list', 'script', 'homedir=', 'tempdir=', 'quiet', 'help'])
    except getopt.error, e:
        sys.stderr.write('Error: %s\n' % str(e))
        usage()
    list = script = 0
    homedir = os.environ.get('NINIX_HOME')
    tempdir = None
    quiet = 0
    for opt, val in options:
        if opt in ['-l', '--list']:
            list = 1
        elif opt in ['-s', '--script']:
            list = script = 1
        elif opt in ['-H', '--homedir']:
            homedir = val
        elif opt in ['-T', '--tempdir']:
            tempdir = val
        elif opt in ['-q', '--quiet']:
            quiet = 1
        else:
            usage()
    if homedir:
        ninix.home.NINIX_HOME = homedir
    if tempdir:
        tempfile.tempdir = os.path.expanduser(tempdir)
    if list:
        list_ghosts(script)
    elif len(argv) == 2:
        update_ghost(argv[0], argv[1], quiet)
    else:
        usage()
        
def list_ghosts(script):
    sys.stderr.write('Searching ghosts...\n')
    config = ninix.home.load_config()
    if config is None:
        sys.stderr.write('Error: ninix home not found\n')
        sys.exit(1)
    ghosts, shells, balloons, plugins, nekoninni, katochan, kinoko = config
    found = 0
    default_path = os.path.join(sys.path[0], 'ninix', 'dll')
    saori_lib = ninix.dll.Library('saori', default_path, ghost=None)
    dll = ninix.dll.Library('shiori', default_path, saori_lib=saori_lib)
    for desc, shiori_dir, use_makoto, surface_set, \
        balloon, prefix, shiori_dll, shiori_name in ghosts:
        name = (shiori_dll, shiori_name)
        homeurl = ''
        shiori = dll.request(name)
        if shiori and shiori.load(shiori_dir):
            if getattr(shiori, 'is_oldtype', None):
                homeurl = shiori.getstring('homeurl')
                shiori.finalize()
            else:
                response = shiori.request('GET SHIORI/3.0\n' \
                                          'ID: homeurl\n' \
                                          'Sender: ninix-aya\n' \
                                          'SecurityLevel: local\n' \
                                          'Charset: ASCII\n\n')
                homeurl = get_value(response)
                shiori.unload()
        if not homeurl:
            continue
        if script:
            if not found:
                print '#!/bin/sh'
            print ''.join(('echo ', '-' * 60))
            print "echo 'Ghost: %s'" % \
                  desc.get('name', '').encode('utf-8', 'ignore')
            print "echo 'Home URL: %s'" % homeurl
            print "echo 'Directory: %s'" % prefix
            print PROGRAM_NAME, homeurl, prefix
        else:
            print '-' * 60
            print 'Ghost:', desc.get('name', '').encode('utf-8', 'ignore')
            print '(%s,%s)' % \
                  (desc.get('sakura.name', '').encode('utf-8', 'ignore'),
                   desc.get('kero.name', '').encode('utf-8', 'ignore'))
            print 'Home URL:', homeurl
            print 'Directory:', prefix
        found = 1
    if not found:
        sys.stderr.write('No ghost found\n')

def get_value(response):
    header = StringIO.StringIO(response)
    result = {}
    while 1:
        line = header.readline()
        if not line:
            break # EOF
        if line.endswith('\n'):
            line = line[:-1]
        line = line.strip()
        if not line:
            continue
        colon = line.find(':')
        if colon >= 0:
            key = line[:colon].strip()
            result[key] = line[colon + 1:].strip()
        else:
            continue
    if result.has_key('Charset'):
        charset = result['Charset']
        try:
            codecs.lookup(charset)
        except:
            sys.stderr.write('Unsupported charset %s' % repr(charset))
        else:
            for key in result.keys():
                result[key] = unicode(result[key], charset, 'ignore')
    if result.has_key('Value'):
        return result['Value']
    else:
        return ''

def update_ghost(homeurl, ghostdir, quiet):
    if not os.path.exists(os.path.expanduser(ghostdir)):
        sys.stderr.write('Error: %s not found\n' % ghostdir)
        sys.exit(1)
    updateman = ninix.update.NetworkUpdate(None)
    updateman.start(homeurl, ghostdir, timeout=60)
    while 1:
        code = updateman.run()
        while 1:
            event = updateman.get_event()
            if not event:
                break
            if quiet:
                continue
            elif event[0] in ['OnUpdateBegin',
                              'OnUpdate.OnMD5CompareComplete',
                              'OnUpdate.OnMD5CompareFailure']:
                print event[0]
            elif event[0] in ['OnUpdateReady',
                              'OnUpdateComplete',
                              'OnUpdateFailure',
                              'OnUpdate.OnDownloadBegin']:
                print '%s (%s)' % event[:2]
        if code == 0:
            break
        if updateman.state == 5 and updateman.schedule:
            print 'File(s) to be updated:'
            for filename, checksum in updateman.schedule:
                print '   ', filename
    updateman.stop()

if __name__ == '__main__':
    main()

==> ninix-aya-3.9.8a.tgz: ninix-aya-3.9.8a/lib/ninix-update.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  ninix-update.py - a command-line network update utility for ninix
#  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
#  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
#  Copyright (C) 2002-2009 by Shyouzou Sugitani <shy@users.sourceforge.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
#  $Id: ninix-update.py,v 1.20 2009/01/04 07:00:23 shy Exp $
#

import getopt
import os
import sys
import tempfile
import StringIO

import codecs
import locale
locale.setlocale(locale.LC_ALL, '')

if 'DISPLAY' in os.environ:
    del os.environ['DISPLAY']

import ninix.home
import ninix.update
import ninix.dll

PROGRAM_NAME = os.environ.get('NINIX_UPDATE', sys.argv[0])

USAGE = '''\
Usage: %s [options] [URL] [directory]
Options:
  -l, --list          show a list of ghosts
  -s, --script        show a list of ghosts as a shell script
  -H, --homedir DIR   ninix home directory (default: ~/.ninix)
  -T, --tempdir DIR   temporary directory (default: /usr/tmp)
  -q, --quiet         suppress most messages
  -h, --help          show this message
'''

def usage():
    raise SystemExit, USAGE % os.path.basename(PROGRAM_NAME)

def main():
    try:
        options, argv = getopt.getopt(sys.argv[1:], 'lsH:T:qh',
            ['list', 'script', 'homedir=', 'tempdir=', 'quiet', 'help'])
    except getopt.error, e:
        sys.stderr.write('Error: %s\n' % str(e))
        usage()
    ghost_list = script = 0
    homedir = os.environ.get('NINIX_HOME')
    tempdir = None
    quiet = 0
    for opt, val in options:
        if opt in ['-l', '--list']:
            ghost_list = 1
        elif opt in ['-s', '--script']:
            ghost_list = script = 1
        elif opt in ['-H', '--homedir']:
            homedir = val
        elif opt in ['-T', '--tempdir']:
            tempdir = val
        elif opt in ['-q', '--quiet']:
            quiet = 1
        else:
            usage()
    if homedir:
        ninix.home.NINIX_HOME = homedir
    if tempdir:
        tempfile.tempdir = os.path.expanduser(tempdir)
    if ghost_list:
        list_ghosts(script)
    elif len(argv) == 2:
        update_ghost(argv[0], argv[1], quiet)
    else:
        usage()
        
def list_ghosts(script):
    sys.stderr.write('Searching ghosts...\n')
    config = ninix.home.load_config()
    if config is None:
        raise SystemExit, 'Error: ninix home not found\n'
    ghosts, shells, balloons, plugins, nekoninni, katochan, kinoko = config
    found = 0
    default_path = os.path.join(sys.path[0], 'ninix', 'dll')
    saori_lib = ninix.dll.Library('saori', default_path)
    dll = ninix.dll.Library('shiori', default_path, saori_lib=saori_lib)
    for desc, shiori_dir, use_makoto, surface_set, \
        balloon, prefix, shiori_dll, shiori_name in ghosts:
        name = (shiori_dll, shiori_name)
        homeurl = ''
        shiori = dll.request(name)
        if shiori and shiori.load(shiori_dir):
            if getattr(shiori, 'is_oldtype', None):
                homeurl = shiori.getstring('homeurl')
                shiori.finalize()
            else:
                response = shiori.request('GET SHIORI/3.0\n' \
                                          'ID: homeurl\n' \
                                          'Sender: ninix-aya\n' \
                                          'SecurityLevel: local\n' \
                                          'Charset: ASCII\n\n')
                homeurl = get_value(response)
                shiori.unload()
        if not homeurl:
            continue
        if script:
            if not found:
                print '#!/bin/sh'
            print ''.join(('echo ', '-' * 60))
            print "echo 'Ghost: %s'" % \
                  desc.get('name', '').encode('utf-8', 'ignore')
            print "echo 'Home URL: %s'" % homeurl
            print "echo 'Directory: %s'" % prefix
            print PROGRAM_NAME, homeurl, prefix
        else:
            print '-' * 60
            print 'Ghost:', desc.get('name', '').encode('utf-8', 'ignore')
            print '(%s,%s)' % \
                  (desc.get('sakura.name', '').encode('utf-8', 'ignore'),
                   desc.get('kero.name', '').encode('utf-8', 'ignore'))
            print 'Home URL:', homeurl
            print 'Directory:', prefix
        found = 1
    if not found:
        sys.stderr.write('No ghost found\n')

def get_value(response):
    header = StringIO.StringIO(response)
    result = {}
    for line in header:
        line = line.strip()
        if not line:
            continue
        if ':' not in line:
            continue
        key, result[key] = [x.strip() for x in line.split(':', 1)]
    if 'Charset' in result:
        charset = result['Charset']
        try:
            codecs.lookup(charset)
        except:
            sys.stderr.write('Unsupported charset %s' % repr(charset))
        else:
            for key in result:
                result[key] = unicode(result[key], charset, 'ignore')
    if 'Value' in result:
        return result['Value']
    else:
        return ''

def update_ghost(homeurl, ghostdir, quiet):
    if not os.path.exists(os.path.expanduser(ghostdir)):
        raise SystemExit, 'Error: %s not found\n' % ghostdir
    updateman = ninix.update.NetworkUpdate(None)
    updateman.start(homeurl, ghostdir, timeout=60)
    while 1:
        code = updateman.run()
        while 1:
            event = updateman.get_event()
            if not event:
                break
            if quiet:
                continue
            elif event[0] in ['OnUpdateBegin',
                              'OnUpdate.OnMD5CompareComplete',
                              'OnUpdate.OnMD5CompareFailure']:
                print event[0]
            elif event[0] in ['OnUpdateReady',
                              'OnUpdateComplete',
                              'OnUpdateFailure',
                              'OnUpdate.OnDownloadBegin']:
                print '%s (%s)' % event[:2]
        if code == 0:
            break
        if updateman.state == 5 and updateman.schedule:
            print 'File(s) to be updated:'
            for filename, checksum in updateman.schedule:
                print '   ', filename
    updateman.stop()

if __name__ == '__main__':
    main()
